#ifndef MAP_H
#define MAP_H

struct Pixel
{
    enum Type {
        BEDROCK, GRASS, TREE, BERRIES, ROCK, IRON_ORE, // nature
        FOOD, WOOD, STONE, IRON, SWORD, // resources
        DUDE, DEAD_DUDE, // humans
        SPAWN, WALL, ROAD, MARK, LIBRARY // buildings
    };
    
    union PixelData {
        int nbRes;
        void* knowledge;
    };
    
    int type;
    PixelData data;
    
    Pixel() : type(GRASS) {}
};

class Map
{
    Pixel **map;
    int m_width;
    int m_height;
    int m_nbTeams;

public:
    Map(int nbTeams, int width, int height = 0);
    ~Map();
    
    int getWidth() const { return m_width; }
    int getHeight() const { return m_height; }
    int getNbTeams() const { return m_nbTeams; }
    
    /**
     * @brief operator [] allows to access a pixel of the map with the following syntax :
     * Pixel &p = map[x][y];
     */
    Pixel* operator[](int i)
        { return map[i]; }
};

typedef void (*GenerateFunction)(Map *map);

#endif // MAP_H
