#ifndef BEHAVIOR_H
#define BEHAVIOR_H

#include <cmath>

#define DUDE_MEMORY_SIZE 128
#define LIBRARY_SIZE 128
#define COM_SIZE 32

enum Dir {
    NORTH,
    EAST,
    SOUTH,
    WEST,
    NO_DIR = -1
};

struct Coord
{
    int x;
    int y;
    
    Coord(const Coord &c) :
        x(c.x), y(c.y) {}
    Coord(int x, int y) :
        x(x), y(y) {}
    Coord(Dir d = NO_DIR)
    {
        switch(d)
        {
            case NORTH : x= 0; y=-1; break;
            case SOUTH : x= 0; y= 1; break;
            case WEST  : x=-1; y= 0; break;
            case EAST  : x= 1; y= 0; break;
            default    : x= 0; y= 0; break;
        }
    }
    Coord operator+(const Coord &c) const
        { return Coord(x+c.x, y+c.y);}
    Coord& operator+=(const Coord &c)
        { x+=c.x; y+=c.y; return *this; }
    Coord operator+(Dir d) const
        { return *this + Coord(d); }
    Coord& operator+=(Dir d)
        { return *this += Coord(d); }
    Coord operator-(const Coord &c) const
        { return Coord(x-c.x, y-c.y); }
    Coord& operator-=(const Coord &c)
        { x-=c.x; y-=c.y; return *this; }
    Coord operator*(const Coord &c) const
        { return Coord(x*c.x, y*c.y); }
    Coord& operator*=(const Coord &c)
        { x*=c.x; y*=c.y; return *this; }
    Coord operator/(const Coord &c) const
        { return Coord(x/c.x, y/c.y); }
    Coord& operator/=(const Coord &c)
        { x/=c.x; y/=c.y; return *this; }
    int dist() const // manhatthan distance
        { return abs(x) + abs(y); }
    int operator~() const
        { return dist(); }
    static dist(const Coord &c1, const Coord &c2)
        { return ~(c1-c2); }
    int dist(const Coord &c) const
        { return dist(*this, c); }
};

struct Com
{
    enum Flags {
        BOOK1 = 0,
        BOOK2 = 1,
        BOOK3 = 2,
        BOOK4 = 3,
        
        READ  = 0,
        WRITE = 4
    };

	int flag;
	char data[COM_SIZE];
};

struct Action
{
    enum Type {
        MOVE,
        ATTACK,
        PICK,
        PUT,
        WORK,
        WAIT,
        COLOR,
        COMMUNICATE
    };
    
	Type type;
	Dir dir;
	Com com_data;
};

class Info
{
    // TODO
public:
    bool success;
    int inventory;
    const Com& getCom();
    int getNear(Dir d);
    int getInfo(Dir d);
};

typedef void (*BehaviorFunction)(Action *action, char *memory, const Info *info);

#endif // BEHAVIOR_H
